==> route53-2.1.0-sources.jar: software/amazon/awssdk/services/route53/paginators/ListResourceRecordSetsPublisher.java <==
/*
 * Copyright 2013-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
 * the License. A copy of the License is located at
 * 
 * http://aws.amazon.com/apache2.0
 * 
 * or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */

package software.amazon.awssdk.services.route53.paginators;

import java.util.Collections;
import java.util.Iterator;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;
import org.reactivestreams.Subscriber;
import software.amazon.awssdk.annotations.Generated;
import software.amazon.awssdk.core.async.SdkPublisher;
import software.amazon.awssdk.core.pagination.async.AsyncPageFetcher;
import software.amazon.awssdk.core.pagination.async.EmptySubscription;
import software.amazon.awssdk.core.pagination.async.PaginatedItemsPublisher;
import software.amazon.awssdk.core.pagination.async.ResponsesSubscription;
import software.amazon.awssdk.services.route53.Route53AsyncClient;
import software.amazon.awssdk.services.route53.model.ListResourceRecordSetsRequest;
import software.amazon.awssdk.services.route53.model.ListResourceRecordSetsResponse;
import software.amazon.awssdk.services.route53.model.ResourceRecordSet;

/**
 * <p>
 * Represents the output for the
 * {@link software.amazon.awssdk.services.route53.Route53AsyncClient#listResourceRecordSetsPaginator(software.amazon.awssdk.services.route53.model.ListResourceRecordSetsRequest)}
 * operation which is a paginated operation. This class is a type of {@link org.reactivestreams.Publisher} which can be
 * used to provide a sequence of {@link software.amazon.awssdk.services.route53.model.ListResourceRecordSetsResponse}
 * response pages as per demand from the subscriber.
 * </p>
 * <p>
 * When the operation is called, an instance of this class is returned. At this point, no service calls are made yet and
 * so there is no guarantee that the request is valid. If there are errors in your request, you will see the failures
 * only after you start streaming the data. The subscribe method should be called as a request to start streaming data.
 * For more info, see {@link org.reactivestreams.Publisher#subscribe(org.reactivestreams.Subscriber)}. Each call to the
 * subscribe method will result in a new {@link org.reactivestreams.Subscription} i.e., a new contract to stream data
 * from the starting request.
 * </p>
 *
 * <p>
 * The following are few ways to use the response class:
 * </p>
 * 1) Using the forEach helper method
 * 
 * <pre>
 * {@code
 * software.amazon.awssdk.services.route53.paginators.ListResourceRecordSetsPublisher publisher = client.listResourceRecordSetsPaginator(request);
 * CompletableFuture<Void> future = publisher.forEach(res -> { // Do something with the response });
 * future.get();
 * }
 * </pre>
 *
 * 2) Using a custom subscriber
 * 
 * <pre>
 * {@code
 * software.amazon.awssdk.services.route53.paginators.ListResourceRecordSetsPublisher publisher = client.listResourceRecordSetsPaginator(request);
 * publisher.subscribe(new Subscriber<software.amazon.awssdk.services.route53.model.ListResourceRecordSetsResponse>() {
 * 
 * public void onSubscribe(org.reactivestreams.Subscriber subscription) { //... };
 * 
 * 
 * public void onNext(software.amazon.awssdk.services.route53.model.ListResourceRecordSetsResponse response) { //... };
 * });}
 * </pre>
 * 
 * As the response is a publisher, it can work well with third party reactive streams implementations like RxJava2.
 * <p>
 * <b>Note: If you prefer to have control on service calls, use the
 * {@link #listResourceRecordSets(software.amazon.awssdk.services.route53.model.ListResourceRecordSetsRequest)}
 * operation.</b>
 * </p>
 */
@Generated("software.amazon.awssdk:codegen")
public class ListResourceRecordSetsPublisher implements SdkPublisher<ListResourceRecordSetsResponse> {
    private final Route53AsyncClient client;

    private final ListResourceRecordSetsRequest firstRequest;

    private final AsyncPageFetcher nextPageFetcher;

    private boolean isLastPage;

    public ListResourceRecordSetsPublisher(Route53AsyncClient client, ListResourceRecordSetsRequest firstRequest) {
        this(client, firstRequest, false);
    }

    private ListResourceRecordSetsPublisher(Route53AsyncClient client, ListResourceRecordSetsRequest firstRequest,
            boolean isLastPage) {
        this.client = client;
        this.firstRequest = firstRequest;
        this.isLastPage = isLastPage;
        this.nextPageFetcher = new ListResourceRecordSetsResponseFetcher();
    }

    @Override
    public void subscribe(Subscriber<? super ListResourceRecordSetsResponse> subscriber) {
        subscriber.onSubscribe(ResponsesSubscription.builder().subscriber(subscriber).nextPageFetcher(nextPageFetcher).build());
    }

    /**
     * Returns a publisher that can be used to get a stream of data. You need to subscribe to the publisher to request
     * the stream of data. The publisher has a helper forEach method that takes in a {@link java.util.function.Consumer}
     * and then applies that consumer to each response returned by the service.
     */
    public final SdkPublisher<ResourceRecordSet> resourceRecordSets() {
        Function<ListResourceRecordSetsResponse, Iterator<ResourceRecordSet>> getIterator = response -> {
            if (response != null && response.resourceRecordSets() != null) {
                return response.resourceRecordSets().iterator();
            }
            return Collections.emptyIterator();
        };
        return PaginatedItemsPublisher.builder().nextPageFetcher(new ListResourceRecordSetsResponseFetcher())
                .iteratorFunction(getIterator).isLastPage(isLastPage).build();
    }

    /**
     * <p>
     * A helper method to resume the pages in case of unexpected failures. The method takes the last successful response
     * page as input and returns an instance of {@link ListResourceRecordSetsPublisher} that can be used to retrieve the
     * consecutive pages that follows the input page.
     * </p>
     */
    private final ListResourceRecordSetsPublisher resume(ListResourceRecordSetsResponse lastSuccessfulPage) {
        if (nextPageFetcher.hasNextPage(lastSuccessfulPage)) {
            return new ListResourceRecordSetsPublisher(client, firstRequest.toBuilder()
                    .startRecordName(lastSuccessfulPage.nextRecordName())
                    .startRecordType(lastSuccessfulPage.nextRecordTypeAsString())
                    .startRecordIdentifier(lastSuccessfulPage.nextRecordIdentifier()).build());
        }
        return new ListResourceRecordSetsPublisher(client, firstRequest, true) {
            @Override
            public void subscribe(Subscriber<? super ListResourceRecordSetsResponse> subscriber) {
                subscriber.onSubscribe(new EmptySubscription(subscriber));
            }
        };
    }

    private class ListResourceRecordSetsResponseFetcher implements AsyncPageFetcher<ListResourceRecordSetsResponse> {
        @Override
        public boolean hasNextPage(final ListResourceRecordSetsResponse previousPage) {
            return previousPage.isTruncated().booleanValue();
        }

        @Override
        public CompletableFuture<ListResourceRecordSetsResponse> nextPage(final ListResourceRecordSetsResponse previousPage) {
            if (previousPage == null) {
                return client.listResourceRecordSets(firstRequest);
            }
            return client.listResourceRecordSets(firstRequest.toBuilder().startRecordName(previousPage.nextRecordName())
                    .startRecordType(previousPage.nextRecordTypeAsString())
                    .startRecordIdentifier(previousPage.nextRecordIdentifier()).build());
        }
    }
}

==> route53-2.1.4-sources.jar: software/amazon/awssdk/services/route53/paginators/ListResourceRecordSetsPublisher.java <==
/*
 * Copyright 2013-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
 * the License. A copy of the License is located at
 * 
 * http://aws.amazon.com/apache2.0
 * 
 * or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */

package software.amazon.awssdk.services.route53.paginators;

import java.util.Collections;
import java.util.Iterator;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;
import org.reactivestreams.Subscriber;
import software.amazon.awssdk.annotations.Generated;
import software.amazon.awssdk.core.async.SdkPublisher;
import software.amazon.awssdk.core.pagination.async.AsyncPageFetcher;
import software.amazon.awssdk.core.pagination.async.EmptySubscription;
import software.amazon.awssdk.core.pagination.async.PaginatedItemsPublisher;
import software.amazon.awssdk.core.pagination.async.ResponsesSubscription;
import software.amazon.awssdk.services.route53.Route53AsyncClient;
import software.amazon.awssdk.services.route53.model.ListResourceRecordSetsRequest;
import software.amazon.awssdk.services.route53.model.ListResourceRecordSetsResponse;
import software.amazon.awssdk.services.route53.model.ResourceRecordSet;

/**
 * <p>
 * Represents the output for the
 * {@link software.amazon.awssdk.services.route53.Route53AsyncClient#listResourceRecordSetsPaginator(software.amazon.awssdk.services.route53.model.ListResourceRecordSetsRequest)}
 * operation which is a paginated operation. This class is a type of {@link org.reactivestreams.Publisher} which can be
 * used to provide a sequence of {@link software.amazon.awssdk.services.route53.model.ListResourceRecordSetsResponse}
 * response pages as per demand from the subscriber.
 * </p>
 * <p>
 * When the operation is called, an instance of this class is returned. At this point, no service calls are made yet and
 * so there is no guarantee that the request is valid. If there are errors in your request, you will see the failures
 * only after you start streaming the data. The subscribe method should be called as a request to start streaming data.
 * For more info, see {@link org.reactivestreams.Publisher#subscribe(org.reactivestreams.Subscriber)}. Each call to the
 * subscribe method will result in a new {@link org.reactivestreams.Subscription} i.e., a new contract to stream data
 * from the starting request.
 * </p>
 *
 * <p>
 * The following are few ways to use the response class:
 * </p>
 * 1) Using the subscribe helper method
 * 
 * <pre>
 * {@code
 * software.amazon.awssdk.services.route53.paginators.ListResourceRecordSetsPublisher publisher = client.listResourceRecordSetsPaginator(request);
 * CompletableFuture<Void> future = publisher.subscribe(res -> { // Do something with the response });
 * future.get();
 * }
 * </pre>
 *
 * 2) Using a custom subscriber
 * 
 * <pre>
 * {@code
 * software.amazon.awssdk.services.route53.paginators.ListResourceRecordSetsPublisher publisher = client.listResourceRecordSetsPaginator(request);
 * publisher.subscribe(new Subscriber<software.amazon.awssdk.services.route53.model.ListResourceRecordSetsResponse>() {
 * 
 * public void onSubscribe(org.reactivestreams.Subscriber subscription) { //... };
 * 
 * 
 * public void onNext(software.amazon.awssdk.services.route53.model.ListResourceRecordSetsResponse response) { //... };
 * });}
 * </pre>
 * 
 * As the response is a publisher, it can work well with third party reactive streams implementations like RxJava2.
 * <p>
 * <b>Note: If you prefer to have control on service calls, use the
 * {@link #listResourceRecordSets(software.amazon.awssdk.services.route53.model.ListResourceRecordSetsRequest)}
 * operation.</b>
 * </p>
 */
@Generated("software.amazon.awssdk:codegen")
public class ListResourceRecordSetsPublisher implements SdkPublisher<ListResourceRecordSetsResponse> {
    private final Route53AsyncClient client;

    private final ListResourceRecordSetsRequest firstRequest;

    private final AsyncPageFetcher nextPageFetcher;

    private boolean isLastPage;

    public ListResourceRecordSetsPublisher(Route53AsyncClient client, ListResourceRecordSetsRequest firstRequest) {
        this(client, firstRequest, false);
    }

    private ListResourceRecordSetsPublisher(Route53AsyncClient client, ListResourceRecordSetsRequest firstRequest,
            boolean isLastPage) {
        this.client = client;
        this.firstRequest = firstRequest;
        this.isLastPage = isLastPage;
        this.nextPageFetcher = new ListResourceRecordSetsResponseFetcher();
    }

    @Override
    public void subscribe(Subscriber<? super ListResourceRecordSetsResponse> subscriber) {
        subscriber.onSubscribe(ResponsesSubscription.builder().subscriber(subscriber).nextPageFetcher(nextPageFetcher).build());
    }

    /**
     * Returns a publisher that can be used to get a stream of data. You need to subscribe to the publisher to request
     * the stream of data. The publisher has a helper forEach method that takes in a {@link java.util.function.Consumer}
     * and then applies that consumer to each response returned by the service.
     */
    public final SdkPublisher<ResourceRecordSet> resourceRecordSets() {
        Function<ListResourceRecordSetsResponse, Iterator<ResourceRecordSet>> getIterator = response -> {
            if (response != null && response.resourceRecordSets() != null) {
                return response.resourceRecordSets().iterator();
            }
            return Collections.emptyIterator();
        };
        return PaginatedItemsPublisher.builder().nextPageFetcher(new ListResourceRecordSetsResponseFetcher())
                .iteratorFunction(getIterator).isLastPage(isLastPage).build();
    }

    /**
     * <p>
     * A helper method to resume the pages in case of unexpected failures. The method takes the last successful response
     * page as input and returns an instance of {@link ListResourceRecordSetsPublisher} that can be used to retrieve the
     * consecutive pages that follows the input page.
     * </p>
     */
    private final ListResourceRecordSetsPublisher resume(ListResourceRecordSetsResponse lastSuccessfulPage) {
        if (nextPageFetcher.hasNextPage(lastSuccessfulPage)) {
            return new ListResourceRecordSetsPublisher(client, firstRequest.toBuilder()
                    .startRecordName(lastSuccessfulPage.nextRecordName())
                    .startRecordType(lastSuccessfulPage.nextRecordTypeAsString())
                    .startRecordIdentifier(lastSuccessfulPage.nextRecordIdentifier()).build());
        }
        return new ListResourceRecordSetsPublisher(client, firstRequest, true) {
            @Override
            public void subscribe(Subscriber<? super ListResourceRecordSetsResponse> subscriber) {
                subscriber.onSubscribe(new EmptySubscription(subscriber));
            }
        };
    }

    private class ListResourceRecordSetsResponseFetcher implements AsyncPageFetcher<ListResourceRecordSetsResponse> {
        @Override
        public boolean hasNextPage(final ListResourceRecordSetsResponse previousPage) {
            return previousPage.isTruncated().booleanValue();
        }

        @Override
        public CompletableFuture<ListResourceRecordSetsResponse> nextPage(final ListResourceRecordSetsResponse previousPage) {
            if (previousPage == null) {
                return client.listResourceRecordSets(firstRequest);
            }
            return client.listResourceRecordSets(firstRequest.toBuilder().startRecordName(previousPage.nextRecordName())
                    .startRecordType(previousPage.nextRecordTypeAsString())
                    .startRecordIdentifier(previousPage.nextRecordIdentifier()).build());
        }
    }
}
